@@ -5,7 +5,7 @@ import io
 
 from datetime import datetime
 from typing import Any, Union
-from cloc.utils import defaultattr
+from cloc.utils import defaultattr, trace
 
 """
 Types still to make:
@@ -26,9 +26,8 @@ class BaseType(object):
     def __init__(self, basetype: Any= None):
         defaultattr(self, 'basetype', basetype)
 
-    def convert(self, value: str):
-        """
-        overload function for new type cast for param input
+    def __call__(self, value: str):
+        """overload __call__ for converting to new type
 
         Args:
             value {str} -- value to convert
@@ -42,9 +41,9 @@ class Choices(BaseType):
         super().__init__(basetype)
         self.choices = set(choices)
 
-    def convert(self, value: str):
+    def __call__(self, value: str):
         if value not in self.choices:
-            raise ValueError(f'Error: {value!r} was not found in choices: {", ".join(self.choices)!r}')
+            trace(f'Error: {value!r} was not found in choices: {", ".join(self.choices)!r}',  TypeError)
         return value
 
 class FileType(BaseType):
@@ -56,11 +55,11 @@ class FileType(BaseType):
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.fobj.close()
 
-    def convert(self, filepath: str):
+    def __call__(self, filepath: str):
         if not os.path.exists(filepath):
-            raise ValueError(f'Error: {filepath!r} does not  exists')
+            trace(f'Error: {filepath!r} does not  exists', TypeError)
         elif not os.path.isfile(filepath):
-            raise ValueError(f'Error: {filepath!r} is not a file')
+            trace(f'Error: {filepath!r} is not a file', TypeError)
         self.fobj = open(filepath, 'r')
         return self.fobj
 
@@ -71,7 +70,7 @@ class IntRangeType(BaseType):
     def __init__(self):
         super().__init__(int)
 
-    def convert(self, value: Union[str, int]):
+    def __call__(self, value: Union[str, int]):
         if isinstance(value, str):
             vals = value.split(',')
             if len(vals) == 1:
@@ -79,11 +78,11 @@ class IntRangeType(BaseType):
             elif len(vals) == 2:
                 return list(range(int(vals[0]), int(vals[1])))
             else:
-                raise ValueError(f'Unable to find a start or stop value based on given: {value!r}')
+                trace(f'Unable to find a start or stop value based on given: {value!r}', TypeError)
         elif isinstance(value, int):
             return list(range(value))
         else:
-            raise TypeError(f'{value!r} was {type(value).__name__!r} and not {"str"!r} or {"int"!r}')
+            trace(f'{value!r} was {type(value).__name__!r} and not {"str"!r} or {"int"!r}', TypeError)
 
 class DateType(BaseType):
     __name__ = 'cloc.Date'
@@ -92,14 +91,14 @@ class DateType(BaseType):
     def __init__(self):
         super().__init__(datetime)
 
-    def convert(self, value: str):
+    def __call__(self, value: str):
         patterns = ('%Y-%m-%d', '%Y-%m-%dT%H:%M:%S', '%Y-%m-%d %H:%M:%S')
         for p in patterns:
             try:
                 return datetime.strptime(value, p)
             except:
                 pass
-        raise ValueError(f'{value!r} did not match any date patterns: {", ".join(patterns)!r}')
+        trace(f'{value!r} was {type(value).__name__!r} and not {"str"!r} or {"int"!r}', TypeError)
 
 class Sha256Type(BaseType):
     __name__ = 'cloc.Sha256'
@@ -108,21 +107,21 @@ class Sha256Type(BaseType):
     def __init__(self):
         super().__init__(str)
 
-    def convert(self, value: str):
+    def __call__(self, value: str):
         if value and isinstance(value, str):
             if os.path.exists(value):
                 if os.path.isfile(value):
                     with open(value, 'r') as fin:
                         return SHA256_PATTERN.findall(fin.read())
                 else:
-                    raise ValueError(f'expected path to be a file, got {value!r}, {type(value).__name__!r}')
+                    trace(f'expected path to be a file, got {value!r}, {type(value).__name__!r}', TypeError)
             else:
                 if SHA256_PATTERN.match(value):
                     return value
                 else:
-                    raise ValueError(f'{value!r} is not a valid sha256')
+                    trace(f'{value!r} is not a valid sha256', TypeError)
         else:
-            raise ValueError(f'expected string for sha256 type conversion, got {value!r} of type {type(value).__name__}')
+            trace(f'expected string for sha256 type conversion, got {value!r} of type {type(value).__name__}')
 
 class UrlType(BaseType):
     __name__ = 'cloc.Url'
@@ -131,10 +130,10 @@ class UrlType(BaseType):
     def __init__(self):
         super().__init__(str)
 
-    def convert(self, value: str):
+    def __call__(self, value: str):
         if URL_PATTERN.match(value):
             return value
-        raise Exception(f'{value!r} is not a valid URL')
+        trace(f'{value!r} is not a valid URL', TypeError)
 
 class JsonType(BaseType):
     __name__ = 'cloc.Json'
@@ -143,13 +142,13 @@ class JsonType(BaseType):
     def __init__(self):
         super().__init__(dict)
 
-    def convert(self, value: Union[str, dict]):
+    def __call__(self, value: Union[str, dict]):
         try:
             if isinstance(value, dict):
                 return value
             return json.loads(value)
         except:
-            raise ValueError(f'{value!r} was not valid JSON')
+            trace(f'{value!r} was not valid JSON', TypeError)
 
 
 """
